==> emelfm2-0.1.7.tar.gz: emelfm2-0.1.7/src/command/complete/e2_complete__files.c <==
/* $Id: e2_complete__files.c 1861 2006-04-25 08:27:44Z tpgww $
 * 
 * Copyright (C) 2004-2005 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "e2_complete.h"
#include <unistd.h>
#include <glob.h>

/**
 * @brief create list files that are valid completions of @a word
  *
 * The string to be completed extends from after the ' ' (if any) preceding
 * the cursor position @a pos, to that position. But if @a pos = 0, the string
 * will be empty
 * If the string in question is from a dirline, the corresponding directory
 * is scanned for completions
 *
 * @param line utf8 string containing the whole line to be completed
 * @param word utf8 string, copy of part of @a line with the 'word' to be completed 
 * @param pos 0-based index of cursor position (characters, not bytes) in @a line
 * @param found pointer to list to record matching items
 * @param flags pointer to bit-flags indicating type of match (dirs or files)
 * @param data discretioanary data pointer
 * @param rt pointer to data struct for the command line whose entry is to be completed
 * @return the number of successful matches
 **/
gint e2_complete_files (gchar *line, gchar *word, gint pos, GList **found,
	E2_CompleteFlags *flags, gpointer data, E2_CommandLineRuntime *rt)
{
	printd (DEBUG, "complete_files (line:%s,word:%s,pos:%d,found:_,flags:%d,data:_)",
		line, word, pos, *flags);

	if (!rt->original	//processing a dir line
		//but couldn't open the current dir shown in the corresponding pane
		//FIXME interpret the entered string, not the current dir string
		&& ! e2_fs_chdir (rt->pane->path))
			return 0;

	//FIXME: use g_pattern_match* through the virtual filesystem when
	//it is available
	gchar *pattern = g_strconcat (word, "*", NULL);
	gchar *pattern_local = F_FILENAME_TO_LOCALE (pattern);

	glob_t matches;
	if (glob (pattern_local, GLOB_MARK, NULL, &matches) != 0)
	{
		g_free (pattern);
		F_FREE (pattern_local);
		return 0;
	}
	else
	{
		guint i;
		gboolean dirs = (*flags & E2_COMPLETE_FLAG_DIRS) && (!(*flags & E2_COMPLETE_FLAG_FILES));
		gboolean files = (*flags & E2_COMPLETE_FLAG_FILES) && (!(*flags & E2_COMPLETE_FLAG_DIRS));
		for (i = 0; i < matches.gl_pathc; i++)
		{
			if (dirs)
			{
				if (g_str_has_suffix (matches.gl_pathv[i], G_DIR_SEPARATOR_S))
					*found = g_list_append (*found, D_FILENAME_FROM_LOCALE (matches.gl_pathv[i]));
			}
			else
				if (files)
			{
				if (!g_str_has_suffix (matches.gl_pathv[i], G_DIR_SEPARATOR_S))
					*found = g_list_append (*found, D_FILENAME_FROM_LOCALE (matches.gl_pathv[i]));
			}
			else
				*found = g_list_append (*found, D_FILENAME_FROM_LOCALE (matches.gl_pathv[i]));
		}
	}

	e2_fs_chdir (curr_pane->path);
	g_free (pattern);
	F_FREE (pattern_local);

	gint count = matches.gl_pathc;
	globfree (&matches);
	return count;
}

==> emelfm2-0.1.8.tar.gz: emelfm2-0.1.8/src/command/complete/e2_complete__files.c <==
/* $Id: e2_complete__files.c 2209 2006-08-05 09:40:50Z tpgww $
 * 
 * Copyright (C) 2004-2005 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "e2_complete.h"
#include <unistd.h>
#include <glob.h>

/**
 * @brief create list of files (and non-files) that are valid completions of @a word
  *
 * The string to be completed extends from after the ' ' (if any) preceding
 * the cursor position @a pos, to that position. But if @a pos = 0, the string
 * will be empty
 * If the string in question is from a dirline, the corresponding directory
 * is scanned for completions
 * For a dirline, @a line = @a word
 *
 * @param line utf8 string containing the whole line to be completed
 * @param word utf8 string, copy of part of @a line with the 'word' to be completed 
 * @param pos 0-based index of cursor position (characters, not bytes) in @a line
 * @param found pointer to list to record matching items
 * @param flags pointer to bit-flags indicating type of match (dirs or files)
 * @param data discretionary data pointer
 * @param pane enumerator of pane to be used for getting dir string 1,2,0=current
 *
 * @return the number of successful matches
 **/
gint e2_complete_files (gchar *line, gchar *word, gint pos, GList **found,
	E2_CompleteFlags *flags, gpointer data, gint pane)
{
	printd (DEBUG, "complete_files (line:%s,word:%s,pos:%d,found:_,flags:%d,data:_)",
		line, word, pos, *flags);

	gchar *path, *pattern, *pattern_local;
	//FIXME vfs: use g_pattern_match* through the virtual filesystem when
	//that's available
	if (!g_path_is_absolute (word))
	{
		switch (pane)
		{
			case 1:
				path = app.pane1_view.dir;
				break;
			case 2:
				path = app.pane2_view.dir;
				break;
//			case 0:
			default:
				path = curr_view->dir;
				break;
		}
		if (!e2_fs_chdir (path))
			return 0;
	}

	gint count;
	pattern = g_strconcat (word, "*", NULL);
	pattern_local = F_FILENAME_TO_LOCALE (pattern);
	glob_t matches;
	if (glob (pattern_local, GLOB_MARK, NULL, &matches) == 0)
	{
		guint i;
		gboolean dirs = (*flags & E2_COMPLETE_FLAG_DIRS) && (!(*flags & E2_COMPLETE_FLAG_FILES));
		gboolean files = (*flags & E2_COMPLETE_FLAG_FILES) && (!(*flags & E2_COMPLETE_FLAG_DIRS));
		for (i = 0; i < matches.gl_pathc; i++)
		{
			if (dirs)
			{
				if (g_str_has_suffix (matches.gl_pathv[i], G_DIR_SEPARATOR_S))
					*found = g_list_append (*found, D_FILENAME_FROM_LOCALE (matches.gl_pathv[i]));
			}
			else
				if (files)
			{
				if (!g_str_has_suffix (matches.gl_pathv[i], G_DIR_SEPARATOR_S))
					*found = g_list_append (*found, D_FILENAME_FROM_LOCALE (matches.gl_pathv[i]));
			}
			else
				*found = g_list_append (*found, D_FILENAME_FROM_LOCALE (matches.gl_pathv[i]));
		}
		count = matches.gl_pathc;
		globfree (&matches);
	}
	else
		count = 0;

	e2_fs_chdir (curr_view->dir);
	g_free (pattern);
	F_FREE (pattern_local);

	return count;
}
